/*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2006 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 *END_COPYRIGHT_BLOCK*/

package edu.rice.cs.drjava.model;

import edu.rice.cs.util.text.AbstractDocumentInterface;

/**
 * Returned to FindMachineDialog with the location of the found string
 * (or -1 if the string was not found) as well as a flag indicating
 * whether the machine wrapped around the end of the document.
 *
 * @version $Id: FindResult.java 4076 2007-01-19 23:01:04Z dlsmith $
 */
public class FindResult {
  private OpenDefinitionsDocument _document;
  private int _foundoffset;
  private boolean _wrapped;
  private boolean _allWrapped;
  
  /**
   * Constructor for a FindResult.
   * @param document the document where the found instance is located
   * @param foundoffset the offset of the instance found
   * @param wrapped {@code true} if the search wrapped to the beginning (or end) of the document
   * @param allWrapped {@code true} if the search wrapped to the start document
   */
  public FindResult(OpenDefinitionsDocument document, int foundoffset, boolean wrapped, boolean allWrapped) {
    _document = document;
    _foundoffset = foundoffset;
    _wrapped = wrapped;
    _allWrapped = allWrapped;
  }
  
  /** Intelligible toString method */
  public String toString() {
    return "FindResult(" + _document + ", " + _foundoffset + ", " + _wrapped + ", " + _allWrapped + ")";
  }
  
  /** Returns the document where the found instance is located */
  public OpenDefinitionsDocument getDocument() { return _document; }
  
  /** Returns the offset of the instance found */ 
  public int getFoundOffset() { return _foundoffset; }
  
  /** Returns true if the search wrapped to the beginning (or end) of the document */
  public boolean getWrapped() { return _wrapped; }
  
  /** Returns true if the search wrapped to the start document.  TODO: change method name to getAllWrapped. */
  public boolean getAllDocsWrapped() { return _allWrapped; }
}
