/*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2005 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 *END_COPYRIGHT_BLOCK*/

package edu.rice.cs.drjava.project;

import java.io.File;

import edu.rice.cs.plt.tuple.Pair;

/** Classes that implement this interface are expected to give information specific to a single document that is
 *  to be saved by the project file builder.  These objects are passed to the builder upon a save.  These objects
 *  should not be cached anywhere so that they may be gc'd.  They are meant for temporary transmition of data.
 */
public interface DocumentInfoGetter {
  
  /** @return the selection with the start and ending locations paired together. The cursor location is at the second
   *  location in the pair while the selection is defined between the two.
   */
  public Pair<Integer,Integer> getSelection();
  
  /** @return the scroll offset of the scroll pane with the first element being the vertical scroll and the second 
   *  being the horizontal scroll
   */
  public Pair<Integer,Integer> getScroll();
  
  /** @return the filename of the document being described by this getter. */
  public File getFile();
  
  /** Returns the package declared in the java code within the document being described
   *  @return the package of this document.  
   */
  public String getPackage();
  
  /** @return true iff the described document is the current active document. */
  public boolean isActive();
  
  /** @return whether the document has a file currently associated with it. */
  public boolean isUntitled();
}