/*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2005 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 END_COPYRIGHT_BLOCK*/

package edu.rice.cs.drjava.ui;

import java.io.File;
import javax.swing.filechooser.FileFilter;
import edu.rice.cs.drjava.DrJava;
import edu.rice.cs.drjava.DrJavaRoot;
import edu.rice.cs.drjava.config.OptionConstants;


/** A file filter for files with extensions ".java" and ".gj". Used in the file choosers for open and save. 
 *  @version $Id: JavaSourceFilter.java 3522 2006-02-11 03:57:00Z rcartwright $
 */
public class JavaSourceFilter extends FileFilter {
  
  /** Returns true if the file's extension matches Java or GJ. */
  public boolean accept(File f) {
    if (f.isDirectory()) {
      return true;
    }
    String extension = getExtension(f);
    if (extension != null) {
      switch (DrJava.getConfig().getSetting(OptionConstants.LANGUAGE_LEVEL)) {
        case (DrJavaRoot.FULL_JAVA): return (extension.equals("java") || extension.equals("j"));
        case (DrJavaRoot.ELEMENTARY_LEVEL): return extension.equals("dj0");
        case (DrJavaRoot.INTERMEDIATE_LEVEL): return extension.equals("dj1");
        case (DrJavaRoot.ADVANCED_LEVEL): return extension.equals("dj2");
      }
    }
    return false;
  }

  /** @return A description of this filter to display. */
  public String getDescription() {
    switch (DrJava.getConfig().getSetting(OptionConstants.LANGUAGE_LEVEL)) {
        case (DrJavaRoot.FULL_JAVA): return "Java source files";
        case (DrJavaRoot.ELEMENTARY_LEVEL): return "Elementary source files (.dj0)";
        case (DrJavaRoot.INTERMEDIATE_LEVEL): return "Intermediate source files (.dj1)";
        case (DrJavaRoot.ADVANCED_LEVEL): return "Advanced source files (.dj2)";
      }
    return "Java source files";
  }

  /* Get the extension of a file. */
  public static String getExtension(File f) {
    String ext = null;
    String s = f.getName();
    int i = s.lastIndexOf('.');
    if (i > 0 && i < s.length() - 1) {
      ext = s.substring(i + 1).toLowerCase();
    }
    return ext;
  }
}



