/*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2005 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 *END_COPYRIGHT_BLOCK*/

package edu.rice.cs.drjava.ui;

import edu.rice.cs.drjava.model.MultiThreadedTestCase;
import edu.rice.cs.drjava.model.OpenDefinitionsDocument;
import edu.rice.cs.drjava.model.SingleDisplayModel;
import edu.rice.cs.drjava.project.DocFile;
import edu.rice.cs.drjava.project.MalformedProjectFileException;
import edu.rice.cs.drjava.project.ProjectFileIR;
import edu.rice.cs.drjava.project.ProjectFileParser;

import edu.rice.cs.plt.io.IOUtil;
import edu.rice.cs.util.FileOpenSelector;
import edu.rice.cs.util.OperationCanceledException;
import edu.rice.cs.util.UnexpectedException;
import edu.rice.cs.util.swing.Utilities;

import java.io.*;
import java.util.Arrays;
import java.util.List;

/** Test functions of Project Facility working through the main frame and model. */
public final class ProjectMenuTest extends MultiThreadedTestCase {

  private volatile MainFrame _frame;
  
  private volatile SingleDisplayModel _model;
  
  /** Temporary files */
  private File _base;
  private File _parent;
  private File _srcDir;
  private File _projDir;
  private File _auxFile;
  private File _projFile;
  private File _file1;
  private File _file2;
  
  private String _file1RelName;
  private String _file2RelName;
  
  /* The reader which reads the test project file */
  BufferedReader reader = null;
  
  private String _projFileText = null;
  
  /** Setup method for each JUnit test case in this Test class. */
  public void setUp() throws Exception {
    super.setUp();
    
    // create temp directory for this test
    _base = new File(System.getProperty("java.io.tmpdir")).getCanonicalFile();
    _parent = IOUtil.createAndMarkTempDirectory("proj", "", _base);
    _srcDir = new File(_parent, "src");
    _srcDir.mkdir(); // create the src directory

    // create project in a directory with an auxiliary file outside of it
    _auxFile = File.createTempFile("aux", ".java").getCanonicalFile();
    File auxFileParent = _auxFile.getParentFile();
    _projFile = new File(_parent, "test.pjt");
    
    _file1 = new File(_srcDir, "test1.java");
    IOUtil.writeStringToFile(_file1, "");  // create dummy file
    _file2 = new File(_srcDir, "test2.java");
    IOUtil.writeStringToFile(_file2, "");// create dummy file
    
//    System.err.println("test1.java and test1.java created");
    
//    // generate the relative path names for the files in the project file
//    String temp = _file1.getParentFile().getCanonicalPath();
//    _file1RelName = _file1.getCanonicalPath().substring(temp.length() + 1);
//    temp = _file2.getParentFile().getCanonicalPath();
//    _file2RelName = _file2.getCanonicalPath().substring(temp.length() + 1);

    _projFileText =
      ";; DrJava project file.  Written with build: 20040623-1933\n" +
      "(source ;; comment\n" +
      "   (file (name \"src/test1.java\")(select 32 32))" +
      "   (file (name \"src/test2.java\")(select 32 32)))";
    
    IOUtil.writeStringToFile(_projFile, _projFileText);

//    Utilities.invokeAndWait(new Runnable() { 
//      public void run() { 
        _frame = new MainFrame();
        _frame.pack();
//      }
//    });

    _model = _frame.getModel();
  }

  public void tearDown() throws Exception {
    IOUtil.deleteOnExitRecursively(_parent);
    _auxFile.delete();
    _frame.dispose();
    _projFile = null;
    _model = null;
    _frame = null;
    super.tearDown();
  }
  
  public void testSetBuildDirectory() throws MalformedProjectFileException, IOException {
    
//    Utilities.showDebug("executing testSetBuildDirectory");
    
    //test set build directory when not in project mode
    File f = new File("");
    _model.setBuildDirectory(f);
    assertEquals("Build directory should not have been set", null, _model.getBuildDirectory());
    
//    System.err.println("Opening Project File");
    Utilities.invokeAndWait(new Runnable() { 
      public void run() { 
        try { _model.openProject(_projFile); }
        catch(Exception e) { throw new UnexpectedException(e); }
      } 
    });
//    System.err.println("Completed Opening Project File");
//    System.err.println("Project documents are: " + _model.getProjectDocuments());
    
    assertEquals("Build directory should not have been set", null, _model.getBuildDirectory());
    
    _model.setBuildDirectory(f);
    assertEquals("Build directory should have been set", f, _model.getBuildDirectory());
    
  }
  
  public void testCloseAllClosesProject()  throws MalformedProjectFileException, IOException {
    
//    Utilities.showDebug("executing testCloseAllClosesProject");
     Utilities.invokeAndWait(new Runnable() { 
      public void run() { 
        try { _model.openProject(_projFile); }
        catch(Exception e) { throw new UnexpectedException(e); }
      } 
    });
    assertTrue("Project should have been opened", _model.isProjectActive());
    
    Utilities.invokeAndWait(new Runnable() { 
      public void run() { 
        try { _frame.closeAll(); }
        catch(Exception e) { throw new UnexpectedException(e); }
      } 
    });

    assertFalse("Project should have been closed", _model.isProjectActive());
  }
  
  public void testSaveProject() throws IOException, MalformedProjectFileException {
    
//     Utilities.showDebug("executing testSaveProject");
    
    Utilities.invokeAndWait(new Runnable() { 
      public void run() { 
        _frame.openProject(new FileOpenSelector() {
          public File[] getFiles() throws OperationCanceledException { return new File[] {_projFile}; }
        });
        
        // open a new file and make it an auxiliary file
        _frame.open(new FileOpenSelector() {
          public File[] getFiles() throws OperationCanceledException {
            return new File[] {_auxFile};
          }
        });
        _frame._moveToAuxiliary();

        _frame.saveProject();
        _frame._closeProject();
      } 
    });
    List<OpenDefinitionsDocument> docs = _model.getOpenDefinitionsDocuments();
    assertEquals("One empty document remaining", 1, docs.size());
    assertEquals("Name is (Untitled)", "(Untitled)", _model.getActiveDocument().toString());
    
    ProjectFileIR pfir = ProjectFileParser.ONLY.parse(_projFile);
    DocFile[] src = pfir.getSourceFiles();
//    System.err.println(Arrays.toString(src));
    DocFile[] aux = pfir.getAuxiliaryFiles();
//    System.err.println(Arrays.toString(aux));
    assertEquals("Number of saved src files", 2, src.length);
    assertEquals("Number of saved aux files", 1, aux.length);
    assertEquals("wrong name for _file2", _file2.getCanonicalPath(), src[1].getCanonicalPath()); // assumes same (not reverse) order
    assertEquals("Wrong name for _file1", _file1.getCanonicalPath(), src[0].getCanonicalPath());
    assertEquals("Wrong aux file", _auxFile.getCanonicalPath(), aux[0].getCanonicalPath());
  }
  
}