/*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2006 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 *END_COPYRIGHT_BLOCK*/

package edu.rice.cs.util;

/** An exception which DrJava throws on an unexpected error.
  * Many times, we have to catch BadLocationExceptions in
  * code that accesses DefinitionDocument, even if we know for a
  * fact that a BadLocationException cannot occur.  In that case,
  * and in other similar cases where we know that an exception should not
  * occur, we throw this on the off chance that something does go wrong.
  * This aids us in debugging the code.
  * @version $Id: UnexpectedException.java 4218 2007-03-26 18:18:58Z rcartwright $
  */
public class UnexpectedException extends RuntimeException {

  private Throwable _value;

   /** Constructs an unexpected exception with <code>value.toString()</code> as it's message. */
  public UnexpectedException(Throwable value) {
    super(value.toString());
    _value = value;
  }

  /** Constructs an unexpected exception for value with custom message string + <code>value.toString()</code>. */
  public UnexpectedException(Throwable value, String msg) {
    super(msg + ": " + value.toString());
    _value = value;
  }
  
  /** Constructs a new RuntimeException to report that unreachable point in code has been reached */
  public UnexpectedException() {
    this(new RuntimeException("Unreachable point in code has been reached!"));
  }

  /** Constructs a new RuntimeException to report specified message */
  public UnexpectedException(String msg) {
    this(new RuntimeException(msg));
  }

  /** Returns the contained exception. */
  public Throwable getCause() { return _value; }
}
