 /*BEGIN_COPYRIGHT_BLOCK
 *
 * This file is part of DrJava.  Download the current version of this project from http://www.drjava.org/
 * or http://sourceforge.net/projects/drjava/
 *
 * DrJava Open Source License
 * 
 * Copyright (C) 2001-2006 JavaPLT group at Rice University (javaplt@rice.edu).  All rights reserved.
 *
 * Developed by:   Java Programming Languages Team, Rice University, http://www.cs.rice.edu/~javaplt/
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
 * documentation files (the "Software"), to deal with the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
 * to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this list of conditions and the 
 *       following disclaimers.
 *     - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the 
 *       following disclaimers in the documentation and/or other materials provided with the distribution.
 *     - Neither the names of DrJava, the JavaPLT, Rice University, nor the names of its contributors may be used to 
 *       endorse or promote products derived from this Software without specific prior written permission.
 *     - Products derived from this software may not be called "DrJava" nor use the term "DrJava" as part of their 
 *       names without prior written permission from the JavaPLT group.  For permission, write to javaplt@rice.edu.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO 
 * THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * WITH THE SOFTWARE.
 * 
 *END_COPYRIGHT_BLOCK*/

package edu.rice.cs.util.docnavigation;

import junit.framework.TestCase;

import edu.rice.cs.util.swing.Utilities;
import edu.rice.cs.drjava.DrJavaTestCase;

import java.io.*;
import java.util.*;

public class JListSortNavigatorTest extends DrJavaTestCase {
  
  protected JListSortNavigator<DummyINavigatorItem> list;
  protected DummyINavigatorItem i1, i2, i3, i4;
  
  public void setUp() throws Exception {
    super.setUp();

    list = new JListSortNavigator<DummyINavigatorItem>();
    
    i1 = new DummyINavigatorItem("item1");
    i2 = new DummyINavigatorItem("item2");
    i3 = new DummyINavigatorItem("item3");
    i4 = new DummyINavigatorItem("item4");
    list.addDocument(i1);
    list.addDocument(i2);
    list.addDocument(i3);
    list.addDocument(i4);
  }
  
  public void testTraversalOps() {
    assertEquals("doc count test", 4, list.getDocumentCount());
    assertSame("getFirst test", i1, list.getFirst());
    assertSame("getLast test", i4, list.getLast());
    
    list.setNextChangeModelInitiated(true);
    list.setActiveDoc(i1);
    
    Utilities.clearEventQueue();
    assertSame("getCurrent test", i1, list.getCurrent());
    assertSame("getNext test 1", i2, list.getNext(i1));
    assertSame("getNext test 2", i3, list.getNext(i2));
    assertSame("getNext test 3", i4, list.getNext(i3));

    assertSame("getPrevious test 1", i3, list.getPrevious(i4));
    assertSame("getPrevious test 2", i2, list.getPrevious(i3));
    assertSame("getPrevious test 3", i1, list.getPrevious(i2));
    
    assertTrue("contains test 1", list.contains(i1));
    assertTrue("contains test 2", list.contains(i2));
    assertTrue("contains test 3", list.contains(i3));
    assertTrue("contains test 4", list.contains(i4));
    
    assertFalse("contains test 5", list.contains(new DummyINavigatorItem("item1")));
    
    Enumeration<DummyINavigatorItem> docs = list.getDocuments();
    DummyINavigatorItem[] docsArray = new DummyINavigatorItem[4];
    for (int i = 0; i < 4; i++) docsArray[i] = docs.nextElement();
    assertTrue("getDocuments test", Arrays.equals(docsArray, new DummyINavigatorItem[] {i1, i2, i3, i4}));
  }
  
  /**
   * Test of getting the list of selected items.
   * Commented out when changes from revision 4171 were reverted.
   */
//  public void testGetSelectedDocuments() {
//    list.clearSelection();
//    list.addSelectionInterval(0, 1);
//    assertEquals("Two items should be selected", 2, list.getSelectionCount());
//    assertEquals("Two items should be selected", 2, list.getDocumentSelectedCount());
//    assertEquals("Zero groups should be selected", 0, list.getGroupSelectedCount());
//    java.util.List<DummyINavigatorItem> l = list.getSelectedDocuments();
//    assertEquals("Two items should be selected", 2, l.size());
//    assertEquals("Wrong item 1", i1, l.get(0));
//    assertEquals("Wrong item 2", i2, l.get(1));
//
//    list.clearSelection();
//    list.addSelectionInterval(0, 3);
//    assertEquals("Four items should be selected", 4, list.getSelectionCount());
//    assertEquals("Four items should be selected", 4, list.getDocumentSelectedCount());
//    assertEquals("Zero groups should be selected", 0, list.getGroupSelectedCount());
//    l = list.getSelectedDocuments();
//    assertEquals("Four items should be selected", 4, l.size());
//    assertEquals("Wrong item 1", i1, l.get(0));
//    assertEquals("Wrong item 2", i2, l.get(1));
//    assertEquals("Wrong item 3", i3, l.get(2));
//    assertEquals("Wrong item 4", i4, l.get(3));
//
//    list.clearSelection();
//    list.addSelectionInterval(0, 1);
//    list.addSelectionInterval(2, 3);
//    assertEquals("Four items should be selected", 4, list.getSelectionCount());
//    assertEquals("Four items should be selected", 4, list.getDocumentSelectedCount());
//    assertEquals("Zero groups should be selected", 0, list.getGroupSelectedCount());
//    l = list.getSelectedDocuments();
//    assertEquals("Four items should be selected", 4, l.size());
//    assertEquals("Wrong item 1", i1, l.get(0));
//    assertEquals("Wrong item 2", i2, l.get(1));
//    assertEquals("Wrong item 3", i3, l.get(2));
//    assertEquals("Wrong item 4", i4, l.get(3));
//    
//    list.clearSelection();
//    assertEquals("Zero items should be selected", 0, list.getSelectionCount());
//    assertEquals("Zero items should be selected", 0, list.getDocumentSelectedCount());
//    assertEquals("Zero groups should be selected", 0, list.getGroupSelectedCount());
//    l = list.getSelectedDocuments();
//    assertEquals("Zero items should be selected", 0, l.size());
//  }
}